@@ -1,5 +1,5 @@
 /* $Id$ */
-/* Copyright (c) 2006-2018 Pierre Pronchery <khorben@defora.org> */
+/* Copyright (c) 2006-2021 Pierre Pronchery <khorben@defora.org> */
 /* This file is part of DeforaOS System libSystem */
 /* All rights reserved.
  *
@@ -32,8 +32,13 @@
 # define LIBSYSTEM_SYSTEM_STRING_H
 
 # include <sys/types.h>
+# include <stdarg.h>
 # include "array.h"
 
+# ifdef __cplusplus
+extern "C" {
+# endif
+
 
 /* String */
 /* types */
@@ -42,15 +47,12 @@ typedef char String;
 ARRAY3(String *, string, String)
 
 
-/* macros */
-/* XXX for compatibility */
-# define string_length(a) string_get_length(a)
-
-
 /* functions */
 String * string_new(String const * string);
 String * string_new_append(String const * string, ...);
+String * string_new_appendv(String const * string, va_list ap);
 String * string_new_format(String const * format, ...);
+String * string_new_formatv(String const * format, va_list ap);
 String * string_new_length(String const * string, size_t length);
 String * string_new_replace(String const * string, String const * what,
 		String const * by);
@@ -66,6 +68,9 @@ int string_set(String ** string, String const * string2);
 
 /* useful */
 int string_append(String ** string, String const * append);
+int string_append_format(String ** string, String const * format, ...);
+int string_append_formatv(String ** string, String const * format, va_list ap);
+
 void string_clear(String * string);
 void string_cut(String * string, size_t length);
 
@@ -88,4 +93,8 @@ size_t string_trim(String * string, String const * which);
 void string_tolower(String * string);
 void string_toupper(String * string);
 
+# ifdef __cplusplus
+}
+# endif
+
 #endif /* !LIBSYSTEM_SYSTEM_STRING_H */
